import React, { ChangeEvent, Component, MouseEvent } from 'react';
import { TodoItem } from './todoItem';

type ItemInputProps = {
  onAddClick: (item: TodoItem) => void;
};
type ItemInputState = {
  newItem: string
};

export class ItemInput extends Component<ItemInputProps, ItemInputState> {

  constructor(props: ItemInputProps) {
    super(props);
    this.state = { newItem: "" };
  }

  render = (): JSX.Element => {
    return (
      <div>
        <label htmlFor='item-input'>New item: </label>
        <input type="text" value={this.state.newItem}
          onChange={this.doNewItemChange} id='item-input' />
        <button type="button"
          onClick={this.doAddClick}>Add Item</button>
      </div>
    );
  }

  doNewItemChange = (evt: ChangeEvent<HTMLInputElement>) => {
    this.setState({ newItem: evt.target.value });
  }

  doAddClick = (_evt: MouseEvent<HTMLButtonElement>) => {
    // Ignore the request if the user hasn't entered a name.
    // (Would be better to disable the button in this case.)
    const name = this.state.newItem.trim();
    if (name.length == 0)
      return;

    this.props.onAddClick({ name: name, completed: false });
    this.setState({ newItem: "" });
  }
}
