import React, { ChangeEvent, Component, MouseEvent } from 'react';
import { TodoItem } from './todoItem';

type TodoItemsProps = {
  items: Array<TodoItem>;
  onItemsChanged: (items: Array<TodoItem>) => void;
};
type TodoItemsState = {};

export class TodoItems extends Component<TodoItemsProps, TodoItemsState> {

  constructor(props: TodoItemsProps) {
    super(props);
  }

  render = (): JSX.Element => {
    return (
      <>
        {this.renderItems()}
        {this.renderInstructions()}
      </>
    );
  }

  renderInstructions = (): JSX.Element => {
    if (this.props.items.length === 0) {
      return <p>No items. Add one to get started!</p>;
    } else {
      return <p>Check the item to mark it completed. Click delete to remove it.</p>;
    }
  }

  renderItems = (): JSX.Element => {
    const items = [];
    for (let i = 0; i < this.props.items.length; i++) {
      const item = this.props.items[i];
      items.push(
        <li key={i}>
          { /* Note: this uses a new type of input called "checkbox", which is a boolean input.
               Unlike "normal" input boxes, it uses checked, not value to store its state.

               Separately, we're passing in an anonymous function, the
               evt => this.doItemToggleComplete(evt, i)
               as the onChange parameter. This creates a new function that takes in one argument,
               the evt, and this function just calls this.doItemToggleComplete but with the extra
               index argument. This lets us have our event handlers have extra parameters!
            */ }
          <input type="checkbox" id={"check" + i} checked={item.completed}
            onChange={evt => this.doItemToggleComplete(evt, i)} />
          <label htmlFor={"check" + i}>
            {/* This ternary strikethroughs the name if the item is completed.
                <s> is them "strikethrough" element. */}
            {item.completed ? <s>{item.name}</s> : item.name}
          </label>{" "} {/* This adds an explicit space between the elements */}
          <button onClick={evt => this.doItemRemove(evt, i)}>Delete</button>
        </li>
      )
    }
    return <ul>{items}</ul>;
  };

  doItemToggleComplete = (_evt: ChangeEvent<HTMLInputElement>, index: number) => {
    const item = this.props.items[index];

    const items = this.props.items.slice(0, index)
      .concat([{ name: item.name, completed: !item.completed }]) // "flipping" the completed field.
      .concat(this.props.items.slice(index + 1));

    this.props.onItemsChanged(items);
  };

  doItemRemove = (_evt: MouseEvent<HTMLButtonElement>, index: number) => {
    const items = this.props.items.slice(0, index)
      .concat(this.props.items.slice(index + 1));

    this.props.onItemsChanged(items);
  }
}
